#include "PyEMClasses.h"

#include "Euler.h"
#include "Vect.h"
#include "VolFrag.h"
#include "Cylinder.h"
#include "PyUtil.h"
#include "PyEMData.h"
#include "PyPickle.h"
#include "PyEuler.h"
#include "PyVect.h"
#include <boost/python.hpp>
#include <boost/python/to_python_converter.hpp>


namespace python = boost::python;

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(print_overloads, print, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Euler_printMatrix, printMatrix, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Euler_diff, diff, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Euler_setAngle1, setAngle, 3, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Euler_setAngle2, setAngle, 4, 5);

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Vect_angle, Vect::angle, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinder_toInventor, toInventor, 0, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinder_toDejaVu, toDejaVu, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinder_toCosec, toCosec, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinder_if_merge, if_merge, 1, 3);

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinders_WriteDejaVu, WriteDejaVu, 1, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinders_WriteCosec, WriteCosec, 1, 7);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinders_OrientationDiff, OrientationDiff, 1, 3);


void export_Euler()
{
    python::class_<Euler> Euler_class("_Euler", "Don't use this class. Use 'Euler' instead.");
    
    python::class_<PyEuler, python::bases<Euler> > PyEuler_class("Euler", "Euler();\nEuler(float a1, float a2, float a3); \nEuler(float a1, float a2, float a3, EulerType type); \nEuler(float e1, float e2, float e3, float e0, EulerType type);");

    PyEuler_class.def(python::init<>());
    PyEuler_class.def(python::init<float, float, float>());
    PyEuler_class.def(python::init<float, float, float, int>());
    PyEuler_class.def(python::init<float, float, float, float, int>());
    PyEuler_class.def(python::init<PyList>());
    
    PyEuler_class.def(python::self * python::self);
    PyEuler_class.def(python::self / python::self);
    PyEuler_class.def(python::self != python::self);
    PyEuler_class.def(python::self == python::self);
    
    PyEuler_class.def("getByType", &PyEuler::getByType,
		      "tuple getByType(EulerType type) \n  Return Euler values in a tuple given a type.");
    
    PyEuler_class.def("setByType", &PyEuler::setByType,
		      "setByType(tuple value, EulerType type) \n  Set Euler values given a tuple and the type.");

    
    Euler_class.def("init", &Euler::init, "init() \n  initialize an Euler instance.");
    Euler_class.def("rectify", &Euler::rectify, "rectify() \n  rectify an Euler instance.");
    Euler_class.def("increment", &Euler::increment,
		    "increment(float delta_alt, float delta_az, float del_phi) \n  increment an Euler in place");
    Euler_class.def("diff", &Euler::diff,
		    Euler_diff("float diff(Euler angle, bool nophi=true) \n  find angle between two direction vectors"));

    Euler_class.def("inverse", &Euler::inverse, "Euler inverse() \n  takes the inverse of an Euler instance");

    Euler_class.def("EulerMult", &Euler::EulerMult,
		    "Euler EulerMult(Euler bc) \n  returns the value of the given rotation");
    Euler_class.def("getMaxSymEl", &Euler::getMaxSymEl,
		    "int getMaxSymEl() \n  gets the order of the group");
    Euler_class.def("resetSym", &Euler::resetSym,
		    "resetSym() \n  initializes symcounter_ without ing 1st element");
    Euler_class.def("FirstSym", &Euler::FirstSym,
		    "Euler FirstSym() \n  initializes");
    Euler_class.def("NextSym", &Euler::NextSym,
		    "Euler NextSym()");
    Euler_class.def("SymN", &Euler::SymN,
		    "Euler SymN(n) \n  returns the n'th symmetric element");
    Euler_class.def("valid", &Euler::valid,
		    "int valid()");
    Euler_class.def("fromMRC", &Euler::fromMRC,
		    "fromMRC(theta, phi, omega) \n  set from MRC angles ");
    Euler_class.def("thetaMRC", &Euler::thetaMRC,
		    "float thetaMRC()");
    Euler_class.def("phiMRC", &Euler::phiMRC, "float phiMRC()");
    Euler_class.def("omegaMRC", &Euler::omegaMRC, "float omegaMRC()");
    Euler_class.def("alt", &Euler::alt, "float alt()");
    Euler_class.def("az", &Euler::az, "float az()");
    Euler_class.def("phi", &Euler::phi, "float phi()");
    Euler_class.def("alpha", &Euler::alpha, "float alpha()");
    Euler_class.def("beta", &Euler::beta, "float beta()");
    Euler_class.def("gamma", &Euler::gamma, "float gamma()");
    Euler_class.def("phiSpider", &Euler::phiSpider, "float phiSpider()");
    Euler_class.def("thetaSpider", &Euler::thetaSpider, "float thetaSpider()");
    Euler_class.def("gammaSpider", &Euler::gammaSpider, "float gammaSpider()");
    Euler_class.def("e0", &Euler::e0, "float e0()");
    Euler_class.def("e1", &Euler::e1, "float e1()");
    Euler_class.def("e2", &Euler::e2, "float e2()");
    Euler_class.def("e3", &Euler::e3, "float e3()");
    Euler_class.def("Q", &Euler::Q, "float Q()");
    Euler_class.def("n1", &Euler::n1, "float n1()");
    Euler_class.def("n2", &Euler::n2, "float n2()");
    Euler_class.def("n3", &Euler::n3, "float n3()");
    Euler_class.def("Qsgi", &Euler::Qsgi, "float Qsgi()");
    Euler_class.def("n1sgi", &Euler::n1sgi, "float n1sgi()");
    Euler_class.def("n2sgi", &Euler::n2sgi, "float n2sgi()");
    Euler_class.def("n3sgi", &Euler::n3sgi, "float n3sgi()");
    Euler_class.def("setSym", &Euler::setSym,
		    "setSym(string symmetry) \n  initialize symcounter to zero");
    Euler_class.def("Print", &Euler::print,
		    print_overloads("print(string id=None, FILE out=stdout)"));
    
    Euler_class.def("printMatrix", &Euler::printMatrix,
		    Euler_printMatrix("printMatrix(string id=None, FILE out=stdout)"));
    
    Euler_class.def("fromLeft", (void (Euler::*)(Euler)) &Euler::fromLeft,
		    "fromLeft(Euler leftEuler) \n  rotates a given rotation  on the left \n\n fromLeft(alt, az, phi) \n  rotates a given rotation  on the left");

    Euler_class.def("fromLeft", (void (Euler::*)(float, float, float)) &Euler::fromLeft,
		    "fromLeft(Euler leftEuler) \nfromLeft(alt, az, phi) \n  rotates a given rotation  on the left");
    
    Euler_class.def("setAngle", (void (Euler::*)(float, float, float, int))0,
		    Euler_setAngle1("setAngle(float a1, float a2, float a3, EulerType type = Euler.EMAN) \nsetAngle(float e1, float e2, float e3, float e0, EulerType type)"));
		    
    Euler_class.def("setAngle", (void (Euler::*)(float, float, float, float, int))0,
		    "setAngle(float a1, float a2, float a3, EulerType type = Euler.EMAN) \nsetAngle(float e1, float e2, float e3, float e0, EulerType type)");

    Euler_class.def_pickle(Euler_pickle_suite());
}


void export_Quater()
{
    python::class_<Quater> Quater_class("Quater", "Quater()\nQuater(Euler e)\nQuater(float x, float y, float z, float q)");
    Quater_class.def(python::init<>());
    Quater_class.def(python::init<Euler&>());
    Quater_class.def(python::init<float, float, float, float>());
    Quater_class.def("fromEuler", &Quater::fromEuler, "fromEuler(Euler e)");
    Quater_class.def("toEuler", &Quater::toEuler, "Euler toEuler()");
    Quater_class.def("Print", &Quater::print,
		     print_overloads("print(string id = None, FILE out = stdout)"));
    
    Quater_class.def_readwrite("nx", &Quater::nx);
    Quater_class.def_readwrite("ny", &Quater::ny);
    Quater_class.def_readwrite("nz", &Quater::nz);
    Quater_class.def_readwrite("Q", &Quater::Q);
}

void export_Vect()
{
    python::class_<Vect> Vect_class("_Vect", "Don't use this class. Use 'Vect' instead.");
    python::class_<PyVect, python::bases<Vect> > PyVect_class("Vect", "Vect()\nVect(float xx, float yy, float zz)");
    
    PyVect_class.def(python::init<>());
    PyVect_class.def(python::init<float, float, float>());
    
    Vect_class.def("normalize", &Vect::normalize, "normalize()");
    Vect_class.def("rotate",  (Vect (Vect::*)(Euler&)) &Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");

    Vect_class.def("rotate",  (Vect (Vect::*)(Quater&)) &Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");
    
    Vect_class.def("rotate",  (Vect (Vect::*)(Vect&, float)) &Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");

    Vect_class.def("rotate",  (Vect (Vect::*)(float, float, float))&Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");
    
    Vect_class.def("rotate",  (Vect (Vect::*)(float, float, float, float))&Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");
    
    Vect_class.def("cross", &Vect::cross, "Vect cross(Vect p)");
    Vect_class.def("getQuater", &Vect::getQuater, "Quater getQuater()");
    Vect_class.def("X", &Vect::X, "float X()");
    Vect_class.def("Y", &Vect::Y, "float Y()");
    Vect_class.def("Z", &Vect::Z, "float Z()");
    Vect_class.def("Alt", &Vect::Alt, "float Alt()");
    Vect_class.def("Az", &Vect::Az, "float Az()");
    Vect_class.def("setX", &Vect::setX, "setX(float xx)");
    Vect_class.def("setY", &Vect::setY, "setY(float yy)");
    Vect_class.def("setZ", &Vect::setZ, "setZ(float zz)");
    Vect_class.def("setXYZ", &Vect::setXYZ, "setXYZ(float xx, float yy, float zz)");
    Vect_class.def("inner", &Vect::inner, "float inner(Vect p)");
    Vect_class.def("angle", &Vect::angle,
		   Vect_angle("float angle(Vect p, Vect y=Vect())"));
    
    Vect_class.def("length", &Vect::length, "float length()");
    Vect_class.def("euler", &Vect::euler, "Euler euler()");
    Vect_class.def("eulerTo", (Euler (Vect::*)(Vect&)) &Vect::eulerTo,
		   "Euler eulerTo(Vect p)\nEuler eulerTo(float x, float y, float z)");
    Vect_class.def("eulerTo", (Euler (Vect::*)(float, float, float))&Vect::eulerTo);
    Vect_class.def("isNeighbor", (bool (Vect::*)(Vect&))&Vect::isNeighbor,
		   "bool isNeighbor(Vect p)\n  26 neighbor connection");
    Vect_class.def("Print", &Vect::print, print_overloads("print(string id=None, FILE out=stdout)"));

    
    PyVect_class.def(python::self * float());
    PyVect_class.def(python::self / float());
    PyVect_class.def(python::self + float());
    PyVect_class.def(python::self + python::self);
    PyVect_class.def(python::self - float());
    PyVect_class.def(python::self - python::self);
    PyVect_class.def(python::self == python::self);
}

void export_VolFrag()
{
    python::class_<VolFrag> VolFrag_class("VolFrag", "VolFrag()\n  construct an empty VolFrag instance");
    VolFrag_class.def(python::init<>());

    VolFrag_class.def("setSize", &VolFrag::setSize, 
		      "setSize(int size)\n  sets object size");
    VolFrag_class.def("getSize", &VolFrag::getSize, 
		      "getSize()");
    VolFrag_class.def("setSizeYZ", &VolFrag::setSizeYZ, 
		      "setSizeYZ(int nx, int y, int z)");
    VolFrag_class.def("setSizeI", &VolFrag::setSizeI, 
		      "setSizeI(int nx, int i)");
    VolFrag_class.def("setupIcos", &VolFrag::setupIcos, 
		      "setupIcos()");
    VolFrag_class.def("zero", &VolFrag::zero, 
		      "zero() \n  zero all elements");
    VolFrag_class.def("validXYZ", &VolFrag::validXYZ, 
		      "int validXYZ(int x, int y, int z)");
    VolFrag_class.def("valueAt", &VolFrag::valueAt, 
		      "float valueAt(int x, int y, int z)");
    VolFrag_class.def("setValue", &VolFrag::setValue, 
		      "setValue(int x, int y, int z, float val)");
    VolFrag_class.def("addValue", &VolFrag::addValue, 
		      "addValue(int x, int y, int z, float val)");
    VolFrag_class.def("multValue", &VolFrag::multValue, 
		      "multValue(int x, int y, int z, float val)");
    VolFrag_class.def("addFrag", &VolFrag::addFrag, 
		      "addFrag(VolFrag src)\n  only within the intersection of the 2 frags");
    VolFrag_class.def("writeFile", &VolFrag::writeFile, 
		      "writeFile(filename)\n  return 0 on success not 0 on failure");
    VolFrag_class.def("readFile", &VolFrag::readFile, 
		      "readFile(filename)\n  return 0 on success not 0 on failure");

}

void export_Cylinder()
{
    python::class_<Cylinder> Cylinder_class("Cylinder", "Cylinder()\n construct an empty Cylinder instance\n\nCylinder(Vect center, Vect axisdir, len = 5.4, r = 2.5, w = 0.9)");
    Cylinder_class.def(python::init<>());
    Cylinder_class.def(python::init<const Vect&, const Vect&, float, float, float>());

    Cylinder_class.def("Center", &Cylinder::Center, 
		       "Vect Center()");
    Cylinder_class.def("Axis", &Cylinder::Axis, 
		       "Vect Axis()");
    Cylinder_class.def("QuaterForAxis", &Cylinder::QuaterForAxis, 
		       "Quater QuaterForAxis()");
    Cylinder_class.def("Xc", &Cylinder::Xc, 
		       "float Xc()");
    Cylinder_class.def("Yc", &Cylinder::Yc, 
		       "float Yc()");
    Cylinder_class.def("Zc", &Cylinder::Zc, 
		       "float Zc()");
    Cylinder_class.def("Alt", &Cylinder::Alt, 
		       "float Alt()");
    Cylinder_class.def("Az", &Cylinder::Az, 
		       "float Az()");
    Cylinder_class.def("Radius", &Cylinder::Radius, 
		       "float Radius()");
    Cylinder_class.def("Length", &Cylinder::Length, 
		       "float Length()");
    Cylinder_class.def("Weight", &Cylinder::Weight, 
		       "float Weight()");
    Cylinder_class.def("StartID", &Cylinder::StartID, 
		       "int StartID()");
    Cylinder_class.def("EndID", &Cylinder::EndID, 
		       "int EndID()");
    Cylinder_class.def("Label", &Cylinder::Label, 
		       "string	Label()");
    Cylinder_class.def("Print", &Cylinder::print,
		       print_overloads("Print(int id = -1, FILE out = stdout)"));

    Cylinder_class.def("toInventor", &Cylinder::toInventor,
		       Cylinder_toInventor("toInventor(FILE out = stdout, id = 0, indent = 1, with_head = false)"));
    Cylinder_class.def("toDejaVu", &Cylinder::toDejaVu,
		       Cylinder_toDejaVu("toDejaVu(FILE out = stdout, id = 0)"));

    Cylinder_class.def("toCosec", &Cylinder::toCosec,
		       Cylinder_toCosec("toCosec(FILE out = stdout, id = 0)"));

    Cylinder_class.def("setCenter", &Cylinder::setCenter, 
		       "setCenter(float x, float y, float z)");
    Cylinder_class.def("setAxis", (void (Cylinder::*)(const Vect&)) &Cylinder::setAxis, 
		       "setAxis(Vect axisdir)\nsetAxis(Euler axisdir)");
    Cylinder_class.def("setAxis", (void (Cylinder::*)(Euler&))&Cylinder::setAxis, 
		       "setAxis(Vect axisdir)\nsetAxis(Euler axisdir)");
    Cylinder_class.def("setRadius", &Cylinder::setRadius, 
		       "setRadius(float r)");
    Cylinder_class.def("setLength", &Cylinder::setLength, 
		       "setLength(float l)");
    Cylinder_class.def("setWeight", &Cylinder::setWeight, 
		       "setWeight(float w)");
    Cylinder_class.def("setStartID", &Cylinder::setStartID, 
		       "setStartID(int i)");
    Cylinder_class.def("setEndID", &Cylinder::setEndID, 
		       "setEndID(int i)");
    Cylinder_class.def("setLabel", &Cylinder::setLabel, 
		       "setLabel(string l)");
    Cylinder_class.def("PlaneAngleDiff", &Cylinder::PlaneAngleDiff, 
		       "float PlaneAngleDiff(Cylinder next)\n  dihedral angle of planes formed by the two axes and center connection vector");
    Cylinder_class.def("AxisAngleDiff", &Cylinder::AxisAngleDiff, 
		       "float AxisAngleDiff(Cylinder next)\n  difference in orientation (axises)");
    Cylinder_class.def("Angle2Vect", &Cylinder::Angle2Vect, 
		       "float Angle2Vect(Vect v)\n  difference in orientation axis of this and the direction vector ");
    Cylinder_class.def("Angle2Point", &Cylinder::Angle2Point, 
		       "float Angle2Point(Vect p)\n  difference in orientation between line passing both P and this center and axis of this");
    Cylinder_class.def("Angle2Center", &Cylinder::Angle2Center, 
		       "float Angle2Center(Cylinder next)\n  difference in orientation between line passing both centers and axis of this");
    Cylinder_class.def("NormDistance", &Cylinder::NormDistance, 
		       "float NormDistance(Cylinder next)\n  shortest distance of the two axes ");
    Cylinder_class.def("CenterDistance", &Cylinder::CenterDistance, 
		       "float	CenterDistance(Cylinder next)\n  distance of the center of next cylinder to the center of this");
    Cylinder_class.def("Distance2Point", &Cylinder::Distance2Point, 
		       "float	Distance2Point(x, y, z)\n  distance of the Point to the center of this cylinder");
    Cylinder_class.def("if_merge", &Cylinder::if_merge,
		       Cylinder_if_merge("bool if_merge(Cylinder next, da_limit = 30*PI/180., distance_limit = 3.2)\n  if next is ok to merge with this"));

    Cylinder_class.def("merge", &Cylinder::merge, 
		       "bool merge(Cylinder next)\n  actually merge both into this");
    Cylinder_class.def("mirror", &Cylinder::mirror, 
		       "mirror()\n  mirror the cylinders");
    Cylinder_class.def("Transform", (void (Cylinder::*)(Euler&, Vect&, Vect&))&Cylinder::Transform, 
		       "Transform(Quater rot, Vect trans, Vect center)\n  transform: rotate then translate\n\nTransform(Euler rot, Vect trans, Vect center)\n  transform: rotate then translate");
    Cylinder_class.def("Transform", (void (Cylinder::*)(Quater&, Vect&, Vect&))&Cylinder::Transform, 
		       "Transform(Quater rot, Vect trans, Vect center)\n  transform: rotate then translate\n\nTransform(Euler rot, Vect trans, Vect center)\n  transform: rotate then translate");


}

void export_Cylinders()
{
    
    python::class_<CylLenComp> CylLenComp_class("CylLenComp", "Cylinder length compare class");
    python::class_<CylWeightComp> CylWeightComp_class("CylWeightComp", "Cylinder weight compare class");
    
    python::class_<Cylinders> Cylinders_class("Cylinders", "A container of Cylinder objects");

    Cylinders_class.def("size", &Cylinders::size, 
			"int size() \n Returns number of Cylinders.");
    Cylinders_class.def("copy", &Cylinders::copy, 
			"Cylinders copy()");
    Cylinders_class.def("subset", &Cylinders::subset, 
			"Cylinders subset(list1, int n)");
    Cylinders_class.def("add", &Cylinders::add, 
			"bool add(Cylinder cyl)");
    
    Cylinders_class.def("ReadListfile", (bool (Cylinders::*)(const char*))0,
			"bool ReadListfile(string in = \"-\");");

    Cylinders_class.def("ReadListfile", (bool (Cylinders::*)(FILE*))0,
			"bool ReadListfile(FILE in = stdin);");

    Cylinders_class.def("WriteListfile", (bool (Cylinders::*)(const char*))0,
			"bool WriteListfile(string out = \"-\");");

    Cylinders_class.def("WriteListfile", (bool (Cylinders::*)(FILE*))0,
			"bool WriteListfile(FILE out = stdout);");

    Cylinders_class.def("ReadInventor", (bool (Cylinders::*)(const char*))0,
			"bool ReadInventor(string in = \"-\");");

    Cylinders_class.def("ReadInventor", (bool (Cylinders::*)(FILE*))0,
			"bool ReadInventor(FILE in = stdin);");

    Cylinders_class.def("WriteInventor", (bool (Cylinders::*)(const char*))0,
			"bool WriteInventor(string out = \"-\");");

    Cylinders_class.def("WriteInventor", (bool (Cylinders::*)(FILE*))0,
			"bool WriteInventor(FILE out = stdout);");

    Cylinders_class.def("WriteVRML", (bool (Cylinders::*)(const char*))0,
			"bool WriteVRML(string out = \"-\");");

    Cylinders_class.def("WriteVRML", (bool (Cylinders::*)(FILE*))0,
			"bool WriteVRML(FILE out = stdout);");

    Cylinders_class.def("ReadDejaVu", (bool (Cylinders::*)(const char*))0,
			"bool ReadDejaVu(string in=\"-\");");

    Cylinders_class.def("ReadDejaVu", (bool (Cylinders::*)(FILE*))0,
			"bool ReadDejaVu(FILE in=stdin);");

    Cylinders_class.def("WriteDejaVu", (bool (Cylinders::*)(FILE*, const char*,const char*, const char*))0,
			Cylinders_WriteDejaVu("bool WriteDejaVu(outfile, mol = '0dum', note = 'dummy protein', pdb = '0dum.pdb')\nbool WriteDejaVu(outfilename, mol = '0dum', note = 'dummy protein', pdb = '0dum.pdb')"));

    Cylinders_class.def("WriteDejaVu", (bool (Cylinders::*)(const char*, const char*,const char*, const char*))0,
			Cylinders_WriteDejaVu("bool WriteDejaVu(outfile, mol = '0dum', note = 'dummy protein', pdb = '0dum.pdb')\nbool WriteDejaVu(outfilename, mol = '0dum', note = 'dummy protein', pdb = '0dum.pdb')"));
    
    Cylinders_class.def("ReadCosec", (bool (Cylinders::*)(const char*))0,
			"bool ReadCosec(string in =\"-\")");

    Cylinders_class.def("ReadCosec", (bool (Cylinders::*)(FILE*))0,
			"bool ReadCosec(FILE in=stdin);");

    Cylinders_class.def("WriteCosec", (bool (Cylinders::*)(FILE*,const char*, const char*, const char*, const char*,const char*, const char*))0,
			Cylinders_WriteCosec("bool WriteCosec(outfile, prog='helixhunter', pdbid='0dum', header='dummy protein', compnd='dummy protein', source='nowhere', author=''\n\nbool WriteCosec(outfilename, prog='helixhunter', pdbid='0dum', header='dummy protein', compnd='dummy protein', source='nowhere', author='')"));
    
    Cylinders_class.def("WriteCosec", (bool (Cylinders::*)(const char*,const char*, const char*,const char*, const char*,const char*, const char*))0,
			Cylinders_WriteCosec("bool WriteCosec(outfile, prog='helixhunter', pdbid='0dum', header='dummy protein', compnd='dummy protein', source='nowhere', author=''\n\nbool WriteCosec(outfilename, prog='helixhunter', pdbid='0dum', header='dummy protein', compnd='dummy protein', source='nowhere', author='')"));
    
    
    Cylinders_class.def("SortLength", &Cylinders::SortLength, 
			"bool SortLength()");
    Cylinders_class.def("SortWeight", &Cylinders::SortWeight, 
			"bool SortWeight()");
    Cylinders_class.def("OrientationDiff", &Cylinders::OrientationDiff,
			Cylinders_OrientationDiff("float OrientationDiff(Cylinders comp, ignoredir=true, uselengthweight=true)\n  average individual cylinders angle diff"));

    Cylinders_class.def("mirror", &Cylinders::mirror, 
			"mirror()");
    Cylinders_class.def("Transform", (void (Cylinders::*)(Quater&, Vect&, Vect&))&Cylinders::Transform, 
			"Transform(Quater rot, Vect trans, Vect center)\n  transform: rotate then translate\n\nTransform(Euler rot, Vect trans, Vect center)\n  transform: rotate then translate");
    Cylinders_class.def("Transform", (void (Cylinders::*)(Euler&, Vect&, Vect&))&Cylinders::Transform, 
			"Transform(Quater rot, Vect trans, Vect center)\n  transform: rotate then translate\n\nTransform(Euler rot, Vect trans, Vect center)\n  transform: rotate then translate");
    Cylinders_class.def("Print", &Cylinders::print, "void print()");
    

}

    
void export_ObjectStat()
{
    python::class_<ObjectStat> ObjectStat_class("ObjectStat", "ObjectStat");

    ObjectStat_class.def("SetObject", (void (ObjectStat::*)(EMData*, const std::vector<Vect*>&))&ObjectStat::SetObject, 
			 "SetObject(EMData map, list obj)\n  obj: a list of Vect ojbects\n\nSetObject(EMData map)");
    ObjectStat_class.def("SetObject", (void (ObjectStat::*)(EMData*))&ObjectStat::SetObject, 
			 "SetObject(EMData map, list obj)\n  obj: a list of Vect ojbects\n\nSetObject(EMData map)");
    ObjectStat_class.def("EigenSortTo", &ObjectStat::EigenSortTo, 
			 "EigenSortTo(ObjectStat obj)");
    ObjectStat_class.def("FitEllipsoid", &ObjectStat::FitEllipsoid, 
			 "FitEllipsoid(sort = true)");
    ObjectStat_class.def("Norm", &ObjectStat::Norm, 
			 "Norm()");
    ObjectStat_class.def("ComputeLongAxis", &ObjectStat::ComputeLongAxis, 
			 "ComputeLongAxis()");
    ObjectStat_class.def("Print", &ObjectStat::print, "print()");

}

void export_Helix()
{
    python::class_<Helix> Helix_class("Helix", "definitons for output format for use in IRIS Explorer module");

    Helix_class.def_readwrite("qx", &Helix::qx);
    Helix_class.def_readwrite("qy", &Helix::qy);
    Helix_class.def_readwrite("qz", &Helix::qz);
    Helix_class.def_readwrite("qa", &Helix::qa);
    Helix_class.def_readwrite("tx", &Helix::tx);
    Helix_class.def_readwrite("ty", &Helix::ty);
    Helix_class.def_readwrite("tz", &Helix::tz);
    Helix_class.def_readwrite("c", &Helix::c);
    Helix_class.def_readwrite("len", &Helix::len);
}


void export_util()
{
    python::def("tmpnam2", tmpnam2, "string tmpnam2()");
    python::def("resolv", resolv, "unsigned long resolv(string host = NULL)");
    python::def("sock_open", sock_open, "int sock_open(unsigned long addr, unsigned short port)");
    python::def("writesock", writesock, "int writesock(int sock, string data, int len)");
    python::def("FSsend", FSsend, "int FSsend(int command, int datalen, string filename, string data,int fileno=0)");
    python::def("maxf", (float (*)(float, float)) maxf, "float maxf(float f1, float f2)");
    python::def("maxf", (float (*)(float, float, float)) maxf, "float maxf(float f1,float f2,float f3)");
    python::def("maxf", (float (*)(float, float, float, float)) maxf, "float maxf(float f1,float f2,float f3,float f4)");
    python::def("minf", (float (*)(float, float, float)) minf, "float minf(float f1,float f2,float f3=1.0e30)");
    python::def("minf", (float (*)(float, float, float, float)) minf, "float minf(float f1,float f2,float f3,float f4)");
    python::def("floor2", floor2, "int floor2(float x)");
    python::def("hypot2", hypot2, "float hypot2(float x, float y)");
    python::def("hypot3", hypot3, "float hypot3(float x,float y,float z)");

    python::def("addHTMLSection", addHTMLSection, "void addHTMLSection(string sourcefile, FILE out, string secid)");
    python::def("error", (void (*)(char*, int))error, "void error(string msg, int pri)");
    python::def("error", (void (*)(int, char*))error, "void error(int pri, string msg)");
  
    python::def("file_lock_wait", file_lock_wait, "int file_lock_wait(FILE file)");
    python::def("file_lock", file_lock, "int file_lock(FILE file)");
    python::def("file_unlock", file_unlock, "void file_unlock(FILE file)");

    python::def("LOG", LOG, "void LOG(int ref, string filename, int mode, string text)");
    
    python::def("LOGbegin", py_LOGbegin, "int LOGbegin(list argv, int ppid=0)");
    
    python::def("LOGerror", LOGerror, "void LOGerror(int ref, int err, string text)");
    
    python::def("appinit", py_appinit, "void appinit(list argv) \n  All applications should call this before parsing the command line");
    python::def("appusage", appusage, "void appusage(string argv0)");
    python::def("frand", frand, "float frand(float min, float max)");
    python::def("grand", grand, "float grand(float mean, float sigma)");
    python::def("cyc", cyc, "int cyc(int i, int n)");
    python::def("interpolate", py_interpolate, "float interpolate(float xl, int n, list x_list, list y_list)");
    python::def("bilin", bilin, "float bilin(float p1, float p2, float p3, float p4, float t, float u)");
    
    python::def("p5peak", p5peak, "float p5peak(float v1, float v2, float v3, float v4,float v5) \n this will interpolate a peak location. v3 is considered to be at x=0. The other points are nearest neighbors returns 0 if not well defined");

    python::def("agauss", agauss, "float agauss(float a, float dx, float dy, float dz, float d)");
    python::def("SQR", SQR, "float SQR(float x)");
    python::def("roteulertb", py_roteulertb, "void py_roteulertb(Float alt, Float az, Float phi, float dx, float dy)");
    python::def("npfa", npfa, "int npfa(int low)");
    python::def("isvalidfft", isvalidfft, "int isvalidfft(int low)");
    python::def("bestfftsize", bestfftsize, "int bestfftsize(int low) \n returns a good fft size >= than low");
    
    python::def("save_data", (void (*)(PyList, PyList, int, const char*)) py_save_data, "void py_save_data(list x_list, list y_list, int n, string filename);");
    
    python::def("save_data", (void (*)(float, float, PyList, int, const char*)) py_save_data, "void py_save_data(float x0, float dx, list y_list, int n, string filename);");
    
    python::def("lsqfit", py_lsqfit, "lsqfit(int n, list dx, list dy, Float m, Float b, int ign0 = 1) \n least squares fit returns slope, m, and intercept, b");
    
    python::def("run", run, "int run(string program);");
}

void export_enums()
{
    python::enum_<ImageType>("Image")
	.value("mrc", IMAGE_MRC)
	.value("spider", IMAGE_SPIDER)
	.value("img", IMAGE_IMAGIC)
	.value("pgm", IMAGE_PGM)
	.value("lst", IMAGE_LST)
	.value("pif", IMAGE_PIF)
	.value("png", IMAGE_PNG)
	.value("dm3", IMAGE_DM3)
	.value("tif", IMAGE_TIF)
	.value("vtk", IMAGE_VTK)
	.value("sal", IMAGE_SAL)
	.value("map", IMAGE_ICOS)
	.value("img3d", IMAGE_IMAGIC3D)
	.value("emim", IMAGE_EMIM)
	.value("single_spider", IMAGE_SINGLE_SPIDER)
	.value("dm2", IMAGE_GATAN)
	.value("amira", IMAGE_AMIRA)
	.value("xplor", IMAGE_XPLOR)
	.value("any", IMAGE_ANY)
	;

    python::enum_<Euler::EulerType>("EulerType")
	.value("eman", Euler::EMAN)
	.value("imagic", Euler::IMAGIC)
	.value("spin", Euler::SPIN)
	.value("quat", Euler::QUAT)
	.value("matrix", Euler::MATRIX)
	.value("sgirot", Euler::SGIROT)
	.value("spider", Euler::SPIDER)
	.value("mrc", Euler::MRC)
	;
}
