// startcsym.C  12/98  Steve Ludtke
// This should be able to generate low quality initial models for icos symmetries

//N starticos
//F This program generates a preliminary 3d model for objects with icosahedral symmetry
//T LR
//1
//P <centered particles>	Raw centered particles to use to build a model
//P <# to keep>	Number of particles to use for each view
//P [imask=<rad>]	Inside mask used to exclude inside regions
//P [nosym]	Skips the initial symmetry search
//U startcsym start.hed 50 sym=c7
//D This program uses a very simple technique to generate a VERY rough 3d
//E model from a set of particle images. It seaches for particles with the
//E best 5-fold rotational symmetry, best 3-fold rotational symmetry, and best
//E 2-fold rotational symmetry. It generates class averages for each of these 
//E three views, then generates a 3D model based on them.
//D Specify the number of particles to use for each view. This number should
//E be at least 10-20, and at most ~10% of the total particle data set.
//E 50-100 is good for typical data sets of 2000 or more particles.
//D This routine will not work well for all data sets. Sometimes it won't
//E work at all, and startAny will have to be used.

#include <stdio.h>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include <math.h>

//#define NITER 8

main(int argc, char *argv[]) {
EMData *im5,*im2,*im3,*f5,*f3,*f2,*c53,*c52,*c32,*vol,*vol2;
int nx,ny,imask=-1,ccl=0,nosym=0;
Euler *a,e;
float phi[4],pr,prt,bpr,bphi,lphi,bnorm,*d,val[181],ctfc=0;
int mode=2,nk,n,i,j,k;
char *sym="icos";
char s[1024];
char *file;

appinit(argc,argv);

if (argc<3) appusage(argv[0]);

nk=atoi(argv[2]);
if (nk<1) { printf("# to keep must be >0\n"); exit(1); }

n=fileCount(argv[1]);
if (n<3) { printf("Input file too small ?!?!\n"); exit(1); }
if (n/3<nk) { printf("<# to keep> should be less than 1/3 number of particles\n"); exit(1); }

for (i=3; i<argc; i++) {	
	if (strncmp(argv[i],"out=",4)==0) file=&argv[i][4];
	else if (strncmp(argv[i],"mode=",5)==0) mode=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"ctfc=",5)==0) ctfc=atof(&argv[i][5]);
	else if (strncmp(argv[i],"imask=",6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp(argv[i],"ccl",3)==0) ccl=1;
	else if (strncmp(argv[i],"nosym",5)==0) nosym=1;
	else { printf("Unknown argument '%s'\n",argv[i]); exit(1); }
}

int Ref=LOGbegin(argc,argv);

if (!nosym) {
	system("/bin/rm -f cls*lst sym.hed sym.img");

// for each symmetry axis (5,3,2 fold), find the best representative
// particles and store them for alignment
	LOG(Ref,argv[1],LOG_INFILE,NULL);
	LOG(Ref,argv[1],LOG_OUTFILE,NULL);
/*	sprintf(s,"symsort %s sym=c5 imask=%d",argv[1],imask);
	system(s);
	sprintf(s,"proc2d %s cls0000.lst last=%d",argv[1],nk-1);
	system(s);

	sprintf(s,"symsort %s sym=c3 imask=%d",argv[1],imask);
	system(s);
	sprintf(s,"proc2d %s cls0001.lst last=%d",argv[1],nk-1);
	system(s);

	sprintf(s,"symsort %s sym=c2 imask=%d",argv[1],imask);
	system(s);
	sprintf(s,"proc2d %s cls0002.lst last=%d",argv[1],nk-1);
	system(s); */

	sprintf(s,"symbest %s cls0000.lst sym=c5 keep=%d imask=%d",argv[1],nk,imask);
	system(s);

	sprintf(s,"symbest %s cls0001.lst sym=c3 keep=%d imask=%d",argv[1],nk,imask);
	system(s);

	sprintf(s,"symbest %s cls0002.lst sym=c2 keep=%d imask=%d",argv[1],nk,imask);
	system(s);

	LOG(Ref,"cls0000.lst",LOG_OUTFILE,NULL);

// Now generate classums for each of the symmetric axes
	sprintf(s,"classalignall keep=1.2 10 precen imask=%d",imask);
	if (ctfc) sprintf(s+strlen(s)," ctfc=%f",ctfc);
	else strcat(s," median");
	system(s);

// Now we actually enforce the symmetry. This could be considered
// cheating a little, but is fine for preliminary images
	system("proc2d classes.hed sym.hed sym=c5 first=0 last=0");
	system("proc2d classes.hed sym.hed sym=c3 first=1 last=1");
	system("proc2d classes.hed sym.hed sym=c2 first=2 last=2");
	LOG(Ref,"sym.hed",LOG_OUTFILE,NULL);

}

// now read in the final symmetric classums from above

LOG(Ref,"sym.hed",LOG_INFILE,NULL);
im5=new EMData;
if (im5->readImage("sym.hed",0)) { printf("classes.hed not created, must have been an error.\n"); exit(1); }
 f5=im5->doFFT();
 f5->toCorner();
im5->gimmeFFT();
 f5->setRAlign(0,0,0);
 f5->setNImg(1);
 f5->getEuler()->setSym(sym);

im3=new EMData;
if (im3->readImage("sym.hed",1)) { printf("classes.hed too short, must have been an error.\n"); exit(1); }
 f3=im3->doFFT();
 f3->toCorner();
im3->gimmeFFT();
 f3->setRAlign(42.29*PI/180.0,0,0);	// ???
 f3->setNImg(1);
 f3->getEuler()->setSym(sym);

im2=new EMData;
if (im2->readImage("sym.hed",2)) { printf("classes.hed too short, must have been an error.\n"); exit(1); }
 f2=im2->doFFT();
 f2->toCorner();
im2->gimmeFFT();
 f2->setRAlign(31.7175*PI/180.0,36.0*PI/180.0,0); // ??? az/phi swapped?
 f2->setNImg(1);
 f2->getEuler()->setSym(sym);

nx=im5->xSize();
ny=im5->ySize();

vol=new EMData;
vol->setup4IS(ny);
vol2=new EMData;
vol2->setup4IS(ny);


printf("Calculate common lines\n");
c52=new EMData;
c53=new EMData;
c32=new EMData;

c52->commonLines(f5,f2,1);
c53->commonLines(f5,f3,1);
c32->commonLines(f3,f2,1);

if (ccl) {
	unlink("ccl.hed");
	unlink("ccl.img");
	c53->writeIMAGIC("ccl.hed",-1);
	c52->writeIMAGIC("ccl.hed",-1);
	c32->writeIMAGIC("ccl.hed",-1);
	LOG(Ref,"ccl.hed",LOG_OUTFILE,NULL);
}

/*
c5=new EMData;
c3=new EMData;
c2=new EMData;

c5->commonLines(f5,f5,1);
c3->commonLines(f3,f3,1);
c2->commonLines(f2,f2,1);
*/


printf("Searching\n");
// 5 -> 3 fold
if (c52->ySize()!=360) exit(1);

d=c53->getDataRO();
phi[0]=0;
phi[1]=0;
pr=0.0;
for (i=0; i<180; i++) {
	for (j=0; j<120; j++) {
		if (d[i+j*360]>pr) { pr=d[i+j*360]; phi[0]=i; phi[1]=j; }
	}
}
c53->doneData();
printf("5 fold at %f\n",phi[0]);
printf("3 fold at %f\n",phi[1]);

// 5 -> 2 fold
d=c52->getDataRO();
phi[2]=0;
phi[3]=0;
pr=0.0;
for (i=0; i<180; i++) {
	for (j=0; j<180; j++) {
		if (d[i+j*360]>pr) { pr=d[i+j*360]; phi[2]=i; phi[3]=j; }
	}
}
c52->doneData();
printf("5 fold at %f\n",phi[2]);
printf("2 fold at %f\n",phi[3]);


// OK now build a model

vol->setup4IS(ny);


/////////////////////////////////////
/*
// 3 fold again
bphi=0;
bpr=1.0e5;

for (phi=30.0*PI/180.0; phi<150.0*PI/180.0; phi+=.25*PI/180.0) {
	vol2->normSlice(f3,f3->alt(),f3->az(),phi,&pr);
	if (pr<bpr) { bpr=pr; bphi=phi; }
}

a=f3->getEuler();
a->setAngle(a->alt(),a->az(),bphi);
for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
	vol->insertSlice(f3,e.alt(),e.az(),e.phi(),mode,1); 			
}
printf("Best 3f pr @ %f (%f)\n",bphi*180.0/PI,bpr);

// 2 fold
bphi=0;
bpr=1.0e5;

for (phi=45.0*PI/180.0; phi<235.0*PI/180.0; phi+=.25*PI/180.0) {
	vol2->normSlice(f2,f2->alt(),f2->az(),phi,&pr);
	if (pr<bpr) { bpr=pr; bphi=phi; }
}

a=f2->getEuler();
a->setAngle(a->alt(),a->az(),bphi);
for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
	vol->insertSlice(f2,e.alt(),e.az(),e.phi(),mode,1); 			
}
printf("Best 2f pr @ %f (%f)\n",bphi*180.0/PI,bpr);


// 5 fold again
a=f5->getEuler();
a->setAngle(a->alt(),a->az(),-phi[0]);
for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
	vol->insertSlice(f5,e.alt(),e.az(),e.phi(),mode,1); 			
}
*/

// 5 fold
bphi=0;
bpr=1.0e5;
//for (phi=36.0*PI/180.0; phi<108.0*PI/180.0; phi+=1.0*PI/180.0) {
//for (phi=0.0*PI/180.0; phi<216.0*PI/180.0; phi+=2.0*PI/180.0) {
for (i=0; i<4; i++) {
	vol->setup4IS(ny);	
	a=f5->getEuler();
	lphi=(90.0*i-phi[0])*PI/180.0;
	lphi=fmod(2.0*PI+lphi,2.0*PI/5.0);
	a->setAngle(0,0,lphi);
	for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
		vol->insertSlice(f5,e.alt(),e.az(),e.phi(),mode,1); 			
	}

	prt=0;
	a->setAngle(0,0,lphi);
	for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
		vol->normSlice(f5,e.alt(),e.az(),e.phi(),&pr); 			
		prt+=pr;
	}
	
printf("%f\t%f\n",lphi*180.0/PI,prt);
	if (prt<bpr) { bpr=prt; bphi=lphi; }

}
printf("Best 5f pr @ %f (%f)\n",bphi*180.0/PI,bpr);

vol->setup4IS(ny);
a=f5->getEuler();
a->setAngle(0,0,bphi);
for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
	vol->insertSlice(f5,e.alt(),e.az(),e.phi(),mode,1); 			
}

// 3 fold
bphi=0;
bpr=1.0e5;
//printf("3 fold-> %f\n",phi[1]);
for (i=0; i<4; i++) {
	a=f3->getEuler();
	lphi=(90.0*i-phi[1])*PI/180.0;
	lphi=fmod(2.0*PI+lphi,2.0*PI/3.0);
	a->setAngle(42.29*PI/180.0,0,lphi);
//	for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
//		vol->insertSlice(f3,e.alt(),e.az(),e.phi(),mode,1); 			
//	}
	prt=0;
	a->setAngle(42.29*PI/180.0,0,lphi);
	for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
		vol->normSlice(f3,e.alt(),e.az(),e.phi(),&pr);
		prt+=pr;
	}
printf("%f\t%f\n",lphi*180.0/PI,prt);
	if (prt<bpr) { bpr=prt; bphi=lphi; }
}
printf("Best 3f pr @ %f (%f)\n",bphi*180.0/PI,bpr);

a=f3->getEuler();
a->setAngle(42.29*PI/180.0,0,bphi);
for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
	vol->insertSlice(f3,e.alt(),e.az(),e.phi(),mode,1); 			
}

// 2 fold
bphi=0;
bpr=1.0e5;
for (i=0; i<4; i++) {
	a=f2->getEuler();
	lphi=(90.0*i-phi[3])*PI/180.0;
	lphi=fmod(2.0*PI+lphi,2.0*PI/2.0);
	a->setAngle(31.7175*PI/180.0,36.0*PI/180.0,lphi);
//	for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
//		vol->insertSlice(f2,e.alt(),e.az(),e.phi(),mode,1); 			
//	}
	prt=0;
	a->setAngle(42.29*PI/180.0,0,lphi);
	for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
		vol->normSlice(f2,e.alt(),e.az(),e.phi(),&pr);
		prt+=pr;
	}
printf("%f\t%f\n",lphi*180.0/PI,prt);
	if (prt<bpr) { bpr=prt; bphi=lphi; }
}
printf("Best 3f pr @ %f (%f)\n",bphi*180.0/PI,bpr);

a=f2->getEuler();
a->setAngle(42.29*PI/180.0,0,bphi);
for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
	vol->insertSlice(f2,e.alt(),e.az(),e.phi(),mode,1); 			
}


vol->doneSlice();
vol->toCorner();
vol2=vol->doIFT();
vol2->applyMask(-1,100+mode);
vol2->applyMask(-1,0);
vol2->writeMRC("threed.0a.mrc");
LOG(Ref,"threed.0a.mrc",LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);
}
