// startoct.C  5/9/00  Steve Ludtke
// This should be able to generate low quality initial models for octahedral symmetry

//N startoct
//F This program generates a preliminary 3d model for objects with octahedral symmetry
//T LR
//1
//P <centered particles>	Raw centered particles to use to build a model
//P <# to keep>	Number of particles to use for each view
//P [nosym]	Skips the initial symmetry search
//U startoct start.hed 50
//D This program uses a very simple technique to generate a VERY rough 3d
//E model from a set of particle images. It seaches for particles with the
//E best 4-fold rotational symmetry. It generates class averages for this 
//E view, then generates a 3D model based on it.
//D Specify the number of particles to use. This number should
//E be at least 10-20, and at most ~10% of the total particle data set.
//E 50-100 is good for typical data sets of 2000 or more particles.
//D This routine will not work well for all data sets. Sometimes it won't
//E work at all, and startAny will have to be used.

#include <stdio.h>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include <math.h>

//#define NITER 8

main(int argc, char *argv[]) {
EMData *im4,*rot4,*rot4c,*vol,*vol2,*f4;
int nx,ny,imask=-1,ccl=0,nosym=0;
float a,ab,qb,ctfc=0,fixrot=0;
int mode=2,nk,n,i,j,k;
char *sym="oct";
char s[1024];
char *file;
Euler *e,e2;

appinit(argc,argv);

if (argc<3) { appusage(argv[0]); exit(0); }

nk=atoi(argv[2]);
if (nk<1) { printf("# to keep must be >0\n"); exit(1); }

n=fileCount(argv[1]);
if (n<3) { printf("Input file too small ?!?!\n"); exit(1); }
if (n/3<nk) { printf("<# to keep> should be less than 1/3 number of particles\n"); exit(1); }

for (i=3; i<argc; i++) {	
	if (strncmp(argv[i],"out=",4)==0) file=&argv[i][4];
	else if (strncmp(argv[i],"ctfc=",5)==0) ctfc=atof(&argv[i][5]);
	else if (strncmp(argv[i],"fixrot=",7)==0) fixrot=atof(&argv[i][7]);
	else if (strncmp(argv[i],"ccl",3)==0) ccl=1;
	else if (strncmp(argv[i],"nosym",5)==0) nosym=1;
	else { printf("Unknown argument '%s'\n",argv[i]); exit(1); }
}

int Ref=LOGbegin(argc,argv);

if (!nosym) {
	system("/bin/rm -f cls*lst sym.hed sym.img");

// for each symmetry axis (5,3,2 fold), find the best representative
// particles and store them for alignment
	LOG(Ref,argv[1],LOG_INFILE,NULL);
	LOG(Ref,argv[1],LOG_OUTFILE,NULL);

	sprintf(s,"symbest %s cls0000.lst sym=c4 keep=%d",argv[1],nk);
	system(s);

	LOG(Ref,"cls0000.lst",LOG_OUTFILE,NULL);

// Now generate classums for each of the symmetric axes
	unlink("classes.hed");
	unlink("classes.img");
//	sprintf(s,"classalignml cls0000.lst 10");
	sprintf(s,"classalign2 cls0000.lst 10 keep=1");
	if (ctfc) sprintf(s+strlen(s)," ctfc=%f",ctfc);
	else strcat(s," median");
	system(s);

// Now we actually enforce the symmetry. 
	unlink("sym.hed");
	unlink("sym.img");
	system("proc2d classes.hed sym.hed first=0 last=0");
	system("proc2d classes.hed sym.hed sym=c4 first=0 last=0");
	LOG(Ref,"sym.hed",LOG_OUTFILE,NULL);

}

// now read in the final symmetric classums from above

LOG(Ref,"sym.hed",LOG_INFILE,NULL);
im4=new EMData;
if (im4->readImage("sym.hed",1)) { printf("classes.hed not created, must have been an error.\n"); exit(1); }

printf("locating best axis\n");
rot4=im4->copy();
for (a=0,i=0; a<PI; a+=PI/90.0,i++) {
	rot4->setRAlign(a);
	rot4->rotateAndTranslate();
	rot4->normalize();
	rot4c=rot4->copy();
	rot4c->vFlip();
	if (rot4c->dot(rot4)>qb) { qb=rot4c->dot(rot4); ab=a; }
	delete rot4c;
}
ab+=fixrot*PI/180.0;

rot4->setRAlign(ab);
rot4->rotateAndTranslate();
rot4->writeIMAGIC("sym.hed",2);
delete rot4;

im4->readImage("sym.hed",2);
f4=im4->doFFT();
f4->toCorner();

ny=im4->ySize();
vol=new EMData;
vol->setup4IS(ny);

e=new Euler(0,0,0);
e->setSym("oct");
for (e2=e->FirstSym(); e2.valid(); e2=e->NextSym()) {
	vol->insertSlice(f4,e2.alt(),e2.az(),e2.phi(),2,1); 			
}


vol->doneSlice();
vol->toCorner();
vol2=vol->doIFT();
vol2->applyMask(-1,100+mode);
vol2->applyMask(-1,0);
vol2->writeMRC("threed.0a.mrc");
LOG(Ref,"threed.0a.mrc",LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);
}
